; CircuitBread Tutorials
; Microcontroller Basics (PIC10F200), Part 7 - Creating a PWM in Assembly
; "by changing the duty cycle of the PWM, we will smoothly adjust the
;  brightness from 0 to full brightness before it drops immediately back to
;  darkness before coming up to full brightness again."
; Link: https://www.circuitbread.com/tutorials/pwm-led-dimming-part-7-microcontroller-basics-pic10f200
; Source code conversion from Microchip MPASM to Microchip XC8 PIC Assembler.

; Include target device specific definitions.
#include <xc.inc>

; Some equates not included in the device specific definitions.
;----- OPTION_REG Bits -----------------------------------------------------
T0CS EQU 0005h
;----- STATUS Bits -----------------------------------------------------
Z    EQU 0002h

; Set the configuration word.
CONFIG WDTE = OFF  ; Watchdog Timer (WDT disabled)
CONFIG CP = OFF    ; Code Protect (Code protection off)
CONFIG MCLRE = OFF ; Master Clear Enable (MCLR disabled, GP3 enabled)

i     EQU    010h    ;define 0x10 register as the PWM loop variable
limit EQU    011h    ;define 0x11 register as the PWM limit
j     EQU    012h    ;define 0x12 register as the delay variable

; NOTE: To make sure the PIC10F200 RC oscillator calibration instruction at
; the 0xFF program memory address is not overwriten place this psect elsewhere
; , for example at the program memory start address: Project Properties >
; pic-as Global Options > Additional Options: -Wl,-pMyCode=0h
    PSECT MyCode,class=CODE,delta=2
INIT:
    MOVLW ~(1 << GPIO_GP1_POSITION)    ;set GP1 as an output
    TRIS GPIO
    CLRF limit         ;Clear the PWM limit
LOOP:
    MOVLW 0FFh         ;Set the initial value of i
    MOVWF i            ;as 0xFF
    BSF GPIO, GPIO_GP1_POSITION      	 ;Set GP1 pin
INT_LOOP:              ;Beginning of the internal PWM loop
    MOVF limit, W      ;Copy the PWM limit value of the W register
    SUBWF i, W         ;Subtract the W from i
    BTFSS STATUS, Z    ;If the result is not 0
    GOTO $ + 2         ;then go to the line 22
    BCF GPIO, GPIO_GP1_POSITION   	 ;else reset the GP1 pin
    CALL DELAY         ;and call the DELAY subroutine
    DECFSZ i, F        ;Decrement the i, and check if the result is 0
    GOTO INT_LOOP      ;If not, return to the internal PWM loop start
    DECF limit, F      ;otherwise change the PWM limit
    GOTO LOOP          ;and return to the main loop

DELAY:                 ;Start DELAY subroutine here
    MOVLW 010h         ;Load initial value for the delay    
    MOVWF j            ;Copy the value to j
DELAY_LOOP:            ;Start delay loop
    DECFSZ j, F        ;Decrement j and check if it is not zero
    GOTO DELAY_LOOP    ;If not, then go to the DELAY_LOOP label
    RETLW 0            ;Else return from the subroutine

    END INIT           ; Program entry point.