; CircuitBread Tutorials
; Microcontroller Basics (PIC10F200), Part 7 - Creating a PWM in Assembly
; "brightness of the LED will smoothly change from the minimum to maximum
;  brightness, and then smoothly from max back down to minimum."
; Link: https://www.circuitbread.com/tutorials/pwm-led-dimming-part-7-microcontroller-basics-pic10f200
; Source code conversion from Microchip MPASM to Microchip XC8 PIC Assembler.

; Include target device specific definitions.
#include <xc.inc>

; Some equates not included in the device specific definitions.
;----- OPTION_REG Bits -----------------------------------------------------
T0CS EQU 0005h
;----- STATUS Bits -----------------------------------------------------
Z    EQU 0002h

; Set the configuration word.
CONFIG WDTE = OFF  ; Watchdog Timer (WDT disabled)
CONFIG CP = OFF    ; Code Protect (Code protection off)
CONFIG MCLRE = OFF ; Master Clear Enable (MCLR disabled, GP3 enabled)

i     EQU    010h    ;define 0x10 register as the PWM loop variable
limit EQU    011h    ;define 0x11 register as the PWM limit
j     EQU    012h    ;define 0x12 register as the delay variable
dir   EQU    013h


; NOTE: To make sure the PIC10F200 RC oscillator calibration instruction at
; the 0xFF program memory address is not overwriten place this psect elsewhere
; , for example at the program memory start address: Project Properties >
; pic-as Global Options > Additional Options: -Wl,-pMyCode=0h
    PSECT MyCode,class=CODE,delta=2
INIT:
    MOVLW ~(1 << GPIO_GP1_POSITION)
    TRIS GPIO
    CLRF limit
    CLRF dir
LOOP:
    MOVLW 0FFh
    MOVWF i
    BSF GPIO, GPIO_GP1_POSITION
INT_LOOP:
    MOVF limit, W
    SUBWF i, W
    BTFSS STATUS, Z
    GOTO $ + 2
    BCF GPIO, GPIO_GP1_POSITION
    CALL DELAY
    DECFSZ i, F
    GOTO INT_LOOP
    BTFSS dir, 0
    GOTO DEC_BRIGHTNESS
    DECFSZ limit, F
    GOTO LOOP
    GOTO TOGGLE_DIR
DEC_BRIGHTNESS:
    INCF limit, F
    MOVLW 0FFh
    SUBWF limit, W
    BTFSS STATUS, Z
    GOTO LOOP
TOGGLE_DIR:
    BTFSS dir, 0
    GOTO SET_DIR
    BCF dir, 0
    GOTO LOOP
SET_DIR:
    BSF dir, 0
    GOTO LOOP

DELAY:
    MOVLW 010h
    MOVWF j
DELAY_LOOP:
    DECFSZ j, F
    GOTO DELAY_LOOP
    RETLW 0

    END INIT           ; Program entry point.